#!/usr/bin/env python
#
# Lara Maia <dev@lara.monster> 2015 ~ 2022
#
# The Steam Tools NG is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of
# the License, or (at your option) any later version.
#
# The Steam Tools NG is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see http://www.gnu.org/licenses/.
#

import sys
# noinspection PyUnresolvedReferences
from logging import Handler, NullHandler
# noinspection PyUnresolvedReferences
from logging.handlers import RotatingFileHandler
from types import TracebackType
from typing import Any, Optional, Type

if sys.platform == 'win32':
    from ctypes import Structure, byref, c_short, windll
    from ctypes.wintypes import DWORD


    class WindowsConsoleCoord(Structure):
        _fields_ = [
            ("x", c_short),
            ("y", c_short),
        ]


    class WindowsConsoleSmallRect(Structure):
        _fields_ = [
            ("left", c_short),
            ("top", c_short),
            ("right", c_short),
            ("bottom", c_short),
        ]


    class WindowsConsoleBufferInfo(Structure):
        _fields_ = [
            ("size", WindowsConsoleCoord),
            ("cursor_position", WindowsConsoleCoord),
            ("attributes", c_short),
            ("window", WindowsConsoleSmallRect),
            ("maximum_window_size", WindowsConsoleCoord),
        ]


    class LowLevelConsoleAPI:
        def __init__(self) -> None:
            self.saved_buffer_info = WindowsConsoleBufferInfo()
            self.screen_buffer = windll.kernel32.GetStdHandle(-11)

        def __enter__(self) -> Any:
            windll.kernel32.GetConsoleScreenBufferInfo(self.screen_buffer, byref(self.saved_buffer_info))
            return self

        def __exit__(self,
                     exception_type: Optional[Type[BaseException]],
                     exception_value: Optional[Exception],
                     traceback: Optional[TracebackType]) -> None:
            windll.kernel32.SetConsoleTextAttribute(self.screen_buffer, self.saved_buffer_info.attributes)

        def set_color(self, color_number: int) -> None:
            windll.kernel32.SetConsoleTextAttribute(self.screen_buffer, color_number)

        def write(self, msg: str) -> None:
            windll.kernel32.WriteConsoleW(self.screen_buffer, msg, len(msg), byref(DWORD(0)), None)


class ColoredStreamHandler(Handler):
    unix_color_map = {
        'INFO': 37,
        'WARNING': 33,
        'ERROR': 35,
        'CRITICAL': 31,
        'DEBUG': 36,
    }

    windows_color_map = {
        'INFO': 1 | 2 | 4,
        'WARNING': 2 | 4 | 8,
        'ERROR': 1 | 4 | 8,
        'CRITICAL': 4 | 8,
        'DEBUG': 2 | 1,
    }

    def emit(self, record: Any) -> None:
        # noinspection PyBroadException
        try:
            msg = record.getMessage().split('\n')

            if sys.platform == 'win32':
                color_number = self.windows_color_map.get(record.levelname, 1 | 2 | 4)

                with LowLevelConsoleAPI() as console:
                    console.set_color(2)
                    console.write(' --> ')

                    console.set_color(color_number)
                    console.write(f'{msg.pop(0)}\r\n')

                    console.set_color(1 | 2 | 4 | 8)
                    for line in msg:
                        console.write(f'{line}\r\n')
            else:
                color_number = self.unix_color_map.get(record.levelname, 37)
                sys.stdout.write('\033[32m --> ')
                sys.stdout.write(f'\033[{color_number}m{msg.pop(0)}\033[m\n')

                sys.stdout.write('\033[1;37m')
                for line in msg:
                    sys.stdout.write(f'{line}\n')
                sys.stdout.write('\033[m')
        except Exception:
            self.handleError(record)



#!/usr/bin/env python
#
# Lara Maia <dev@lara.monster> 2015 ~ 2022
#
# The Steam Tools NG is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of
# the License, or (at your option) any later version.
#
# The Steam Tools NG is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see http://www.gnu.org/licenses/.
#
import asyncio
import logging

from gi.repository import Gtk

from . import utils, settings, login
from .. import i18n, config

log = logging.getLogger(__name__)
_ = i18n.get_translation


# noinspection PyUnusedLocal
class AdvancedSettingsDialog(Gtk.Dialog):
    def __init__(
            self,
            parent_window: Gtk.Window,
            application: Gtk.Application,
            toggle_button: Gtk.ToggleButton,
    ) -> None:
        super().__init__(use_header_bar=True)
        self.parent_window = parent_window
        self.application = application
        self.toggle_button = toggle_button

        self.set_default_size(600, 150)
        self.set_title(_('Advanced Settings'))
        self.set_transient_for(parent_window)
        self.set_modal(True)
        self.set_destroy_with_parent(True)
        self.set_resizable(False)

        self.gtk_settings_class = Gtk.Settings.get_default()

        content_area = self.get_content_area()
        content_area.set_margin_start(10)
        content_area.set_margin_end(10)

        content_grid = Gtk.Grid()
        content_grid.set_row_spacing(10)
        content_grid.set_column_spacing(10)
        content_area.append(content_grid)

        login_section = utils.Section("login", _("Advanced Login Settings"))
        content_grid.attach(login_section, 0, 0, 1, 2)

        shared_secret = login_section.new_item('shared_secret', _("Shared Secret:"), Gtk.Entry, 0, 0)
        shared_secret.connect('changed', settings.on_setting_changed)

        token_item = login_section.new_item("token", _("Token:"), Gtk.Entry, 0, 1)
        token_item.connect("changed", settings.on_setting_changed)

        token_secure_item = login_section.new_item("token_secure", _("Token Secure:"), Gtk.Entry, 0, 2)
        token_secure_item.connect("changed", settings.on_setting_changed)

        identity_secret = login_section.new_item('identity_secret', _("Identity Secret:"), Gtk.Entry, 2, 0)
        identity_secret.connect('changed', settings.on_setting_changed)

        deviceid = login_section.new_item('deviceid', _("Device ID:"), Gtk.Entry, 2, 1)
        deviceid.connect('changed', settings.on_setting_changed)

        steamid_item = login_section.new_item("steamid", _("Steam ID:"), Gtk.Entry, 2, 2)
        steamid_item.set_input_purpose(Gtk.InputPurpose.DIGITS)
        steamid_item.connect("changed", settings.on_digit_only_setting_changed)

        reset_button = utils.AsyncButton()
        reset_button.set_label(_("Reset Everything (USE WITH CAUTION!!!)"))
        reset_button.set_name("reset_button")
        reset_button.connect("clicked", self.on_reset_clicked)
        login_section.grid.attach(reset_button, 0, 9, 4, 1)

        self.connect('response', lambda dialog, response_id: self._exit())

    def _exit(self) -> None:
        # self.toggle_button.set_active(False)
        # FIXME: enabled plugins must be updated
        self.parent_window.destroy()
        self.destroy()

    async def on_reset_clicked(self, button: Gtk.Button) -> None:
        login_dialog = login.LoginDialog(self.parent_window, self.application)
        login_dialog.status.info(_("Reseting... Please wait!"))
        login_dialog.set_deletable(False)
        login_dialog.user_details_section.hide()
        login_dialog.advanced_login.hide()
        login_dialog.show()
        await asyncio.sleep(3)

        config.config_file.unlink(missing_ok=True)

        config.parser.clear()
        config.init()
        self.parent_window.destroy()

        login_dialog.status.info(_("Waiting"))
        login_dialog.user_details_section.show()
        login_dialog.advanced_login.show()

        self.destroy()



#!/usr/bin/env python
#
# Lara Maia <dev@lara.monster> 2015 ~ 2022
#
# The Steam Tools NG is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of
# the License, or (at your option) any later version.
#
# The Steam Tools NG is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see http://www.gnu.org/licenses/.
#
import asyncio
import binascii
import codecs
import getpass
import logging
import tempfile
from typing import TYPE_CHECKING

import aiohttp

from stlib import login
from . import utils
from .. import i18n, config

if TYPE_CHECKING:
    from . import cli

log = logging.getLogger(__name__)
_ = i18n.get_translation


# noinspection PyUnusedLocal
class Login:
    def __init__(self, cli_: 'cli.SteamToolsNG', mobile_login: bool = True) -> None:
        self.cli = cli_
        self.mobile_login = mobile_login
        self.has_user_data = False
        self.captcha_gid = -1
        self._username = ''
        self.__password = ''
        self._mail_code = ''
        self._steam_code = ''
        self._captcha_text = ''

    @property
    def username(self) -> str:
        return self._username

    def set_password(self, encrypted_password: str) -> None:
        key = codecs.decode(encrypted_password, 'rot13')
        raw = codecs.decode(key.encode(), 'base64')
        self.__password = raw.decode()

    @property
    def mail_code(self) -> str:
        return self._mail_code

    @property
    def steam_code(self) -> str:
        return self._steam_code

    @property
    def captcha_text(self) -> str:
        return self._captcha_text

    @property
    def shared_secret(self) -> str:
        return config.parser.get("login", "shared_secret")

    @property
    def identity_secret(self) -> str:
        return config.parser.get("login", "identity_secret")

    async def do_login(self, auto: bool) -> None:
        utils.set_console(info=_("Retrieving user data"))

        if auto:
            self._username = config.parser.get("login", "account_name")
            encrypted_password = config.parser.get("login", "password")
            self.set_password(encrypted_password)

        if not self.username or not self.__password:
            user_input = utils.safe_input(_("Please, write your username"))
            assert isinstance(user_input, str), "Safe input is returning bool when it should return str"
            config.new("login", "account_name", user_input)
            self._username = user_input

            self.__password = getpass.getpass(_("Please, write your password (IT'S HIDDEN, and will be encrypted)"))
            password_key = codecs.encode(self.__password.encode(), 'base64')
            encrypted_password = codecs.encode(password_key.decode(), 'rot13')
            config.new("login", "password", encrypted_password)

        _login_session = login.Login.get_session(0)
        _login_session.username = self.username
        _login_session.password = self.__password

        kwargs = {'emailauth': self.mail_code, 'mobile_login': self.mobile_login}

        # no reason to send captcha_text if no gid is found
        if self.captcha_gid != -1:
            kwargs['captcha_text'] = self.captcha_text
            kwargs['captcha_gid'] = self.captcha_gid
            # if login fails for any reason, gid must be unset
            # CaptchaError exception will reset it if needed
            self.captcha_gid = -1

        if not self.shared_secret or not self.identity_secret:
            log.warning(_("No shared secret found. Trying to log-in without two-factor authentication."))
            # self.code_item.show()

        kwargs['shared_secret'] = self.shared_secret
        kwargs['authenticator_code'] = self.steam_code

        log.info(_("Logging in"))

        while True:
            try:
                login_data = await _login_session.do_login(**kwargs)
            except login.MailCodeError:
                user_input = utils.safe_input(_("Write code received by email"))
                assert isinstance(user_input, str), "safe_input is returning bool when it should return str"
                self._mail_code = user_input
                await self.do_login(True)
            except login.TwoFactorCodeError:
                user_input = utils.safe_input(_("Write Steam Code"))
                assert isinstance(user_input, str), "safe_input is returning bool when it should return str"
                self._steam_code = user_input
                await self.do_login(True)
            except login.LoginBlockedError:
                log.error(_(
                    "Your network is blocked!\n"
                    "It'll take some time until unblocked. Please, try again later\n"
                ))
                self.cli.on_quit()
            except login.CaptchaError as exception:
                self.captcha_gid = exception.captcha_gid

                log.info(_("Steam server is requesting a captcha code."))

                with tempfile.NamedTemporaryFile(buffering=0, prefix='stng_', suffix='.captcha') as temp_file:
                    temp_file.write(exception.captcha)
                    temp_file.flush()

                    user_input = utils.safe_input(
                        _("Open {} in an image view and write captcha code that it shows").format(temp_file.name),
                    )

                assert isinstance(user_input, str)
                self._captcha_text = user_input
                await self.do_login(True)
            except binascii.Error:
                log.error(_("shared secret is invalid!"))
                self.cli.on_quit()
            except login.LoginError as exception:
                log.error(str(exception))
                config.remove('login', 'token')
                config.remove('login', 'token_secure')
                config.remove('login', 'oauth_token')
                self.cli.on_quit()
            except (aiohttp.ClientError, ValueError):
                log.error(_("Check your connection. (server down?)"))
                await asyncio.sleep(15)
                continue
            else:
                new_configs = {}

                if "shared_secret" in login_data.auth:
                    new_configs["shared_secret"] = login_data.auth["shared_secret"]
                elif self.shared_secret:
                    new_configs["shared_secret"] = self.shared_secret

                if "identity_secret" in login_data.auth:
                    new_configs['identity_secret'] = login_data.auth['identity_secret']
                elif self.identity_secret:
                    new_configs["identity_secret"] = self.identity_secret

                if login_data.oauth:
                    new_configs['steamid'] = login_data.oauth['steamid']
                    new_configs['token'] = login_data.oauth['wgtoken']
                    new_configs['token_secure'] = login_data.oauth['wgtoken_secure']
                    new_configs['oauth_token'] = login_data.oauth['oauth_token']
                else:
                    new_configs['steamid'] = login_data.auth['transfer_parameters']['steamid']
                    new_configs['token'] = login_data.auth['transfer_parameters']['webcookie']
                    new_configs['token_secure'] = login_data.auth['transfer_parameters']['token_secure']

                for key, value in new_configs.items():
                    config.new("login", key, value)

                self.has_user_data = True

            break



#!/usr/bin/env python
#
# Lara Maia <dev@lara.monster> 2015 ~ 2022
#
# The Steam Tools NG is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of
# the License, or (at your option) any later version.
#
# The Steam Tools NG is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see http://www.gnu.org/licenses/.
#
import time
from dataclasses import dataclass
from functools import cache, wraps
from typing import Tuple, Any, Callable


@dataclass
class ModuleData:
    display: str = ''
    status: str = ''
    info: str = ''
    error: str = ''
    level: Tuple[int, int] = (0, 0)
    action: str = ''
    raw_data: Any = None


def time_offset_cache(ttl: int = 60) -> Callable[[Callable[[], int]], Callable[[], int]]:
    def wrapper(function_: Any) -> Callable[[], int]:
        function_ = cache(function_)
        function_.time_base = time.time()

        @wraps(function_)
        def wrapped() -> int:
            if time.time() >= function_.time_base + ttl:
                function_.cache_clear()
                function_.time_base = time.time()

            time_raw = function_()

            if function_.cache_info().currsize == 0:
                assert isinstance(time_raw, int)
                return time_raw

            function_.time_offset = function_.time_base - time_raw
            return round(time.time() + function_.time_offset)

        return wrapped

    return wrapper



#!/usr/bin/env python
#
# Lara Maia <dev@lara.monster> 2015 ~ 2022
#
# The Steam Tools NG is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of
# the License, or (at your option) any later version.
#
# The Steam Tools NG is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see http://www.gnu.org/licenses/.
#
import asyncio
import contextlib
import functools
import logging
import ssl
import sys
from pathlib import Path
from typing import Optional, Any, Callable

import aiohttp

import stlib
from stlib import plugins, universe, login, community, webapi, internals
from . import authenticator, utils
from . import login as cli_login
from .. import i18n, config, core

log = logging.getLogger(__name__)
_ = i18n.get_translation


def while_running(function: Callable[..., Any]) -> Callable[..., Any]:
    @functools.wraps(function)
    async def wrapper(self: 'SteamToolsNG', *args: Any, **kwargs: Any) -> None:
        while True:
            await function(self, *args, **kwargs)

            if self.stop:
                self.on_quit()

    return wrapper


# noinspection PyUnusedLocal
class SteamToolsNG:
    def __init__(self, module_name: str, module_options: str) -> None:
        self.module_name = module_name
        self.stop = False
        self.custom_gameid = 0

        if module_name in ['cardfarming', 'fakerun'] and not stlib.steamworks_available:
            log.critical(_(
                "{} module has been disabled because you have "
                "a stlib built without SteamWorks support. To enable it again, "
                "reinstall stlib with SteamWorks support"
            ).format(module_name))
            sys.exit(1)

        if module_name in ['steamtrades', 'steamgifts']:
            if not plugins.has_plugin(module_name):
                log.critical(_(
                    "{0} module has been disabled because you don't "
                    "have {0} plugin installed. To enable it again, "
                    "install the {0} plugin."
                ).format(module_name))
                sys.exit(1)

        try:
            if module_name == 'fakerun' and not module_options:
                raise ValueError

            for option in module_options:
                self.stop = True
                if option != 'oneshot':
                    self.custom_gameid = int(option)
        except ValueError:
            logging.critical("Wrong command line params!")
            sys.exit(1)

        self.api_url = config.parser.get("steam", "api_url")

    @property
    def steamid(self) -> Optional[universe.SteamId]:
        steamid = config.parser.getint("login", "steamid")

        if steamid:
            return universe.generate_steamid(steamid)

        return None

    # FIXME: https://github.com/python/asyncio/pull/465
    def run(self) -> None:
        loop = asyncio.new_event_loop()
        asyncio.set_event_loop(loop)
        task = loop.create_task(self.async_activate())
        task.add_done_callback(utils.safe_task_callback)

        with contextlib.suppress(KeyboardInterrupt):
            loop.run_forever()

    async def do_login(self, *, block: bool = True, auto: bool = False) -> None:
        login_session = cli_login.Login(self)
        await login_session.do_login(auto)

    async def async_activate(self) -> None:
        ssl_context = ssl.SSLContext()

        if hasattr(sys, 'frozen'):
            _executable_path = Path(sys.executable).parent
            ssl_context.load_verify_locations(cafile=_executable_path / 'etc' / 'cacert.pem')

        tcp_connector = aiohttp.TCPConnector(ssl=ssl_context)
        http_session = aiohttp.ClientSession(raise_for_status=True, connector=tcp_connector)
        login_session = await login.Login.new_session(0, api_url=self.api_url, http_session=http_session)

        log.info(_("Logging on Steam. Please wait!"))

        token = config.parser.get("login", "token")
        token_secure = config.parser.get("login", "token_secure")

        if not token or not token_secure or not self.steamid:
            await self.do_login()

        login_session.restore_login(self.steamid, token, token_secure)

        try:
            if await login_session.is_logged_in(self.steamid):
                log.info("Steam login Successful")
            else:
                await self.do_login(auto=True)
        except aiohttp.ClientError as exception:
            log.exception(str(exception))
            log.error(_("Check your connection. (server down?)"))
            await asyncio.sleep(10)
            return  # FIXME: RETRY ###

        community_session = await community.Community.new_session(0, api_url=self.api_url)

        try:
            api_key = await community_session.get_api_key()
            log.debug(_('SteamAPI key found: %s'), api_key)

            if api_key[1] != 'Steam Tools NG':
                raise AttributeError
        except AttributeError:
            log.warning(_('Updating your SteamAPI dev key'))
            await asyncio.sleep(3)
            await community_session.revoke_api_key()
            await asyncio.sleep(3)
            api_key = await community_session.register_api_key('Steam Tools NG')

            if not api_key:
                raise ValueError(_('Something wrong with your SteamAPI dev key'))

        await webapi.SteamWebAPI.new_session(0, api_key=api_key[0], api_url=self.api_url)
        await internals.Internals.new_session(0)

        if self.module_name in ['steamtrades', 'steamgifts']:
            plugin = plugins.get_plugin(self.module_name)
            await plugin.Main.new_session(0)

        log.debug(_("Initializing module %s"), self.module_name)
        module = getattr(self, f"run_{self.module_name}")
        task = asyncio.create_task(module())
        log.debug(_("Adding a new callback for %s"), task)
        task.add_done_callback(utils.safe_task_callback)

    async def run_add_authenticator(self) -> None:
        authenticator_config = authenticator.NewAuthenticator(self)
        await authenticator_config.add_authenticator()

    @while_running
    async def run_steamguard(self) -> None:
        steamguard = core.steamguard.main()

        async for module_data in steamguard:
            utils.set_console(module_data)

    @while_running
    async def run_cardfarming(self) -> None:
        cardfarming = core.cardfarming.main(self.steamid, self.custom_gameid)

        async for module_data in cardfarming:
            utils.set_console(module_data)

    @while_running
    async def run_fakerun(self) -> None:
        fakerun = core.fakerun.main(self.steamid, self.custom_gameid)

        async for module_data in fakerun:
            utils.set_console(module_data)

    @while_running
    async def run_steamtrades(self) -> None:
        steamtrades = core.steamtrades.main()

        async for module_data in steamtrades:
            utils.set_console(module_data)

            if module_data.action == "login":
                await self.do_login(auto=True)
                continue

    @while_running
    async def run_steamgifts(self) -> None:
        steamgifts = core.steamgifts.main()

        async for module_data in steamgifts:
            utils.set_console(module_data)

            if module_data.action == "login":
                await self.do_login(auto=True)
                continue

    @staticmethod
    def on_quit(exit_code: int = 0) -> None:
        loop = asyncio.get_running_loop()
        loop.stop()
        sys.exit(exit_code)



#!/usr/bin/env python
#
# Lara Maia <dev@lara.monster> 2015 ~ 2022
#
# The Steam Tools NG is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of
# the License, or (at your option) any later version.
#
# The Steam Tools NG is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see http://www.gnu.org/licenses/.
#
import asyncio
import atexit
import contextlib
import random
import sys
import time
from pathlib import Path
from subprocess import call
from typing import AsyncGenerator, Dict, Optional, Any

import aiohttp
from psutil import Popen, NoSuchProcess

from stlib import webapi, client, universe, community
from . import utils
from .. import i18n, config

_ = i18n.get_translation


# FIXME: Workaround for keyboard lag when running games on GUI on Windows
# FIXME: https://gitlab.gnome.org/GNOME/gtk/-/issues/2015
# FIXME: L36:79
def killall(process: 'Popen') -> None:
    if not process.is_running():
        return

    for child in process.children(recursive=True):
        with contextlib.suppress(NoSuchProcess):
            child.kill()
            child.wait()

    process.kill()
    process.wait()


class SteamAPIExecutorWorkaround:
    def __init__(self, appid: int, *args: Any, **kwargs: Any) -> None:
        self.process = None
        self.appid = appid

        if sys.platform == 'win32':
            if getattr(sys, 'frozen', False):
                executor_path = [str(Path(sys.executable).parent / 'steam-api-executor.exe'), str(self.appid)]
            else:
                executor_path = [sys.executable, '-m', 'steam_tools_ng.steam_api_executor', str(self.appid)]

            self.process = Popen(executor_path, creationflags=0x08000000)
            atexit.register(killall, self.process)
        else:
            self.executor = _SteamAPIExecutor(appid, *args, **kwargs)

    def shutdown(self, *args: Any, **kwargs: Any) -> None:
        if sys.platform == 'win32':
            killall(self.process)
        else:
            self.executor.shutdown(*args, **kwargs)


_SteamAPIExecutor = client.SteamAPIExecutor
client.SteamAPIExecutor = SteamAPIExecutorWorkaround


# FIXME: L36:79
# FIXME: ---- #

async def while_has_cards(
        steamid: universe.SteamId,
        badge: community.Badge,
) -> AsyncGenerator[utils.ModuleData, None]:
    webapi_session = webapi.SteamWebAPI.get_session(0)
    community_session = community.Community.get_session(0)

    while badge.cards != 0:
        first_wait = config.parser.getint("cardfarming", "first_wait")
        default_wait = config.parser.getint("cardfarming", "default_wait")
        min_wait = config.parser.getint("cardfarming", "min_wait")

        try:
            game_list = await webapi_session.get_owned_games(steamid, appids_filter=[badge.appid])
            game_info = game_list[0]
        except aiohttp.ClientError:
            yield utils.ModuleData(error=_("Check your connection. (server down?)"), info=_("Waiting Changes"))
            await asyncio.sleep(10)
            continue

        if game_info.playtime_forever * 60 >= first_wait:
            wait_offset = random.randint(default_wait, int(default_wait / 100 * 125))
        else:
            wait_offset = first_wait - game_info.playtime_forever * 60

        try:
            executor = client.SteamAPIExecutor(badge.appid)
        except AttributeError:
            yield utils.ModuleData(action='ignore', info=_("Invalid game id {}. Ignoring.").format(badge.appid))
            break
        except ProcessLookupError:
            yield utils.ModuleData(error=_("Steam Client is not running."), info=_("Waiting Changes"))
            await asyncio.sleep(15)
            continue

        for past_time in range(wait_offset):
            current_time = round((wait_offset - past_time) / 60)
            current_time_size = _('minutes')

            if current_time <= 1:
                current_time = wait_offset - past_time
                current_time_size = _('seconds')

            yield utils.ModuleData(
                display=str(badge.appid),
                info=_("{} {} for {}").format(current_time, current_time_size, badge.name),
                status=_("Running {}").format(badge.name),
                level=(past_time, wait_offset),
                raw_data=executor,
                action="check",
            )
            await asyncio.sleep(1)

        executor.shutdown()
        wait_offset = random.randint(min_wait, int(min_wait / 100 * 125))
        for past_time in range(wait_offset):
            yield utils.ModuleData(
                display=str(badge.appid),
                info=f"{_('Updating drops')} ({badge.name})",
                status=_("Game paused"),
                level=(past_time, wait_offset),
            )
            await asyncio.sleep(1)

        while True:
            try:
                cards = await community_session.get_card_drops_remaining(steamid, badge.appid)
            except aiohttp.ClientError:
                yield utils.ModuleData(error=_("Check your connection. (server down?)"), info=_("Waiting Changes"))
                await asyncio.sleep(10)
            except community.BadgeError:
                yield utils.ModuleData(error=_("Steam Server is busy"), info=_("Waiting Changes"))
                await asyncio.sleep(20)
            else:
                break

        # noinspection PyProtectedMember
        badge = badge._replace(cards=cards)

    utils.ModuleData(
        display=str(badge.appid),
        info=_("{} ({})").format(_("Done"), badge.name),
    )


async def main(steamid: universe.SteamId, custom_game_id: int = 0) -> AsyncGenerator[utils.ModuleData, None]:
    reverse_sorting = config.parser.getboolean("cardfarming", "reverse_sorting")
    max_concurrency = config.parser.getint("cardfarming", "max_concurrency")
    invisible = config.parser.getboolean("cardfarming", "invisible")
    community_session = community.Community.get_session(0)

    try:
        badges = sorted(
            await community_session.get_badges(steamid),
            key=lambda badge_: badge_.cards,  # type: ignore
            reverse=reverse_sorting
        )
    except aiohttp.ClientError:
        yield utils.ModuleData(error=_("Check your connection. (server down?)"), info=_("Waiting Changes"))
        await asyncio.sleep(10)
        return

    if not badges or (custom_game_id and custom_game_id not in [badge.appid for badge in badges]):
        yield utils.ModuleData(error=_("No more cards to drop."), info=_("Waiting Changes"))
        await asyncio.sleep(random.randint(500, 1200))
        return

    generators = {}

    if invisible:
        call(f'{config.file_manager} "steam://friends/status/invisible"')

    for badge in badges:
        yield utils.ModuleData(
            display=str(badge.appid),
            status=_("Loading {}").format(badge.name),
        )

        if custom_game_id and badge.appid != custom_game_id:
            yield utils.ModuleData(info=_("Skipping {}").format(badge.appid))
            continue

        generators[badge.appid] = while_has_cards(steamid, badge)

    tasks: Dict[int, Optional[asyncio.Task[Any]]] = {}
    executors = {}
    semaphore = asyncio.Semaphore(max_concurrency)
    last_update = 0

    while True:
        for appid in generators.keys():
            progress_coro = anext(generators[appid])
            assert asyncio.iscoroutine(progress_coro)

            if appid not in tasks:
                if semaphore.locked():
                    break

                await semaphore.acquire()
                tasks[appid] = asyncio.create_task(progress_coro)

            if not tasks[appid]:
                continue

            current_task = tasks[appid]
            assert isinstance(current_task, asyncio.Task)

            if current_task.done():
                semaphore.release()

                if current_task.exception():
                    if isinstance(current_task.exception(), StopAsyncIteration):
                        tasks[appid] = None
                        continue

                    current_exception = current_task.exception()
                    assert isinstance(current_exception, BaseException)
                    raise current_exception

                await semaphore.acquire()
                tasks[appid] = asyncio.create_task(progress_coro)

        if not any(tasks.values()):
            break

        await asyncio.wait([task for task in tasks.values() if task], return_when=asyncio.FIRST_COMPLETED)

        for appid, task in tasks.items():
            if task and task.done() and not task.exception():
                data: utils.ModuleData = task.result()

                if data.action == 'check':
                    executors[appid] = data.raw_data

                if int(time.time()) > last_update + 3:
                    current_running = len(tasks)
                    total_remaining = len(generators) - len([task for task in tasks.values() if not task])
                    yield utils.ModuleData(
                        display=' : '.join([str(appid) for appid in tasks.keys()]),
                        info=data.info,
                        status=_('Running {} from {} remaining').format(current_running, total_remaining),
                        level=data.level,
                        raw_data=executors.values(),
                        action=data.action,
                    )
                    last_update = int(time.time())



#!/usr/bin/env python
#
# Lara Maia <dev@lara.monster> 2015 ~ 2022
#
# The Steam Tools NG is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of
# the License, or (at your option) any later version.
#
# The Steam Tools NG is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see http://www.gnu.org/licenses/.
#

import sys

from importlib.metadata import version, PackageNotFoundError
from pathlib import Path

try:
    __version__ = version(__package__)
except PackageNotFoundError:  # Freezed
    # noinspection PyPackageRequirements
    from win32com.client import Dispatch

    parser = Dispatch('Scripting.FileSystemObject')
    working_directory = Path(sys.executable).parent.resolve()
    version_raw = parser.GetFileVersion(working_directory / Path(sys.executable).name)
    __version__ = str(version_raw).rpartition('.')[0]



#!/usr/bin/env python
#
# Lara Maia <dev@lara.monster> 2015 ~ 2022
#
# The Steam Tools NG is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of
# the License, or (at your option) any later version.
#
# The Steam Tools NG is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see http://www.gnu.org/licenses/.
#
import asyncio
import contextlib
import logging
from typing import Any, Dict, List, Tuple, Union

from gi.repository import Gtk

from stlib import universe, community
from . import utils
from .. import config, i18n

log = logging.getLogger(__name__)
_ = i18n.get_translation


# noinspection PyUnusedLocal
class FinalizeDialog(Gtk.Dialog):
    def __init__(
            self,
            parent_window: Gtk.Window,
            application: Gtk.Application,
            action: str,
            model: Gtk.TreeModel,
            iter_: Union[Gtk.TreeIter, bool, None] = False,
    ) -> None:
        super().__init__(use_header_bar=True)
        self.parent_window = parent_window
        self.application = application
        self.community_session = community.Community.get_session(0)

        if action == "allow":
            self.action = _("accept")
        else:
            self.action = _("cancel")

        self.raw_action = action
        self.model = model
        self.iter = iter_

        self.header_bar = self.get_header_bar()
        self.header_bar.set_show_title_buttons(False)

        self.set_default_size(300, 60)
        self.set_title(_('Finalize Confirmation'))
        self.set_transient_for(self.parent_window)
        self.set_modal(True)
        self.set_destroy_with_parent(True)
        self.set_resizable(False)

        self.content_area = self.get_content_area()
        self.content_area.set_orientation(Gtk.Orientation.VERTICAL)
        self.content_area.set_spacing(10)
        self.content_area.set_margin_start(10)
        self.content_area.set_margin_end(10)
        self.content_area.set_margin_top(10)
        self.content_area.set_margin_bottom(10)

        self.status = utils.SimpleStatus()
        self.content_area.append(self.status)

        self.yes_button = Gtk.Button()
        self.yes_button.set_label(_("Continue"))
        self.yes_button.connect("clicked", self.on_yes_button_clicked)
        self.header_bar.pack_end(self.yes_button)

        self.no_button = Gtk.Button()
        self.no_button.set_label(_("Cancel"))
        self.no_button.connect("clicked", lambda button: self.destroy())
        self.header_bar.pack_start(self.no_button)

        if self.iter is None or not model:
            self.status.error(_("You must select something"))
            self.header_bar.set_show_title_buttons(True)
            self.yes_button.hide()
            self.no_button.hide()
        elif self.iter is False:
            self.status.info(
                _("Do you really want to {} ALL confirmations?\nIt can't be undone!").format(self.action.upper())
            )
        else:
            self.set_size_request(600, 400)

            self.give_label = Gtk.Label()
            self.content_area.append(self.give_label)

            self.receive_label = Gtk.Label()
            self.content_area.append(self.receive_label)

            self.give_label.set_markup(
                utils.markup(
                    _("You are trading the following items with {}:").format(self.model[self.iter][4]),
                    color='blue',
                )
            )

            self.status.info(_("Do you really want to {} that?\nIt can't be undone!").format(self.action.upper()))

            self.grid = Gtk.Grid()
            self.content_area.append(self.grid)

            self.give_tree = utils.SimpleTextTree(_("You will give"), fixed_width=300, model=Gtk.ListStore)
            self.grid.attach(self.give_tree, 0, 0, 1, 1)

            self.arrow = Gtk.Image()
            self.arrow.set_from_icon_name('emblem-synchronizing-symbolic')
            self.grid.attach(self.arrow, 1, 0, 1, 1)

            self.receive_tree = utils.SimpleTextTree(_("You will receive"), fixed_width=300, model=Gtk.ListStore)
            self.grid.attach(self.receive_tree, 2, 0, 1, 1)

            utils.copy_childrens(self.model, self.give_tree.store, self.iter, 3)
            utils.copy_childrens(self.model, self.receive_tree.store, self.iter, 5)

        self.connect('response', lambda dialog, response_id: self.destroy())

    def on_yes_button_clicked(self, button: Gtk.Button) -> None:
        button.hide()
        self.no_button.hide()

        with contextlib.suppress(AttributeError):
            self.give_label.hide()
            self.give_tree.hide()
            self.arrow.hide()
            self.receive_tree.hide()

        self.set_size_request(0, 0)
        self.header_bar.set_show_title_buttons(False)
        self.parent_window.confirmation_tree.lock = True
        loop = asyncio.get_event_loop()
        task: asyncio.Task[Union[Dict[str, Any], List[Tuple[Gtk.TreeIter, Dict[str, Any]]]]]

        if self.iter:
            task = loop.create_task(self.finalize())
        else:
            task = loop.create_task(self.batch_finalize())

        task.add_done_callback(self.on_task_finish)

    def on_task_finish(self, task: asyncio.Task[Any]) -> None:
        exception = task.exception()

        if exception and not isinstance(exception, asyncio.CancelledError):
            try:
                current_exception = task.exception()
                assert isinstance(current_exception, BaseException)
                raise current_exception
            except Exception as exception:
                stack = task.get_stack()

                for frame in stack:
                    log.error("%s at %s", type(exception).__name__, frame)

                log.error("Steam Server is slow. (%s)", str(exception))

                self.status.error(
                    _("Steam Server is slow. Please, try again.")
                )

                self.header_bar.set_show_title_buttons(True)
                self.yes_button.hide()
        else:
            self.parent_window.confirmation_tree.lock = False
            self.destroy()

    async def finalize(self, keep_iter: bool = False) -> Dict[str, Any]:
        identity_secret = config.parser.get("login", "identity_secret")
        steamid = config.parser.getint("login", "steamid")
        deviceid = config.parser.get("login", "deviceid")

        self.status.info(_("Waiting Steam Server (OP: {})").format(self.model[self.iter][1]))

        # steam confirmation server isn't reliable
        for i in range(2):
            result = await self.community_session.send_confirmation(
                identity_secret,
                universe.generate_steamid(steamid),
                deviceid,
                self.model[self.iter][0],
                self.model[self.iter][2],
                self.raw_action,
            )
            await asyncio.sleep(0.5)

        if not keep_iter:
            try:
                self.model.remove(self.iter)
            except IndexError:
                log.debug(_("Unable to remove tree path %s (already removed?). Ignoring."), self.iter)

        # noinspection PyUnboundLocalVariable
        assert isinstance(result, dict)
        return result

    async def batch_finalize(self) -> List[Tuple[Gtk.TreeIter, Dict[str, Any]]]:
        results = []
        batch_status = utils.Status(20, "")
        batch_status.set_pausable(False)
        self.content_area.append(batch_status)
        batch_status.get_label_widget().hide()
        batch_status.set_info(_("Waiting Steam Server response"))
        batch_status.show()
        self.status.hide()

        confirmation_count = len(self.model)

        for index in range(confirmation_count):
            self.iter = self.model[index].iter

            batch_status.set_display(self.model[self.iter][1])

            try:
                batch_status.set_level(index, confirmation_count)
            except KeyError:
                batch_status.set_level(0, 0)

            result = await self.finalize(True)
            results.append((self.iter, result))

        batch_status.hide()
        self.status.show()
        self.status.info(_("Updating tree"))
        for iter_, result in results:
            self.model.remove(iter_)

        return results



#!/usr/bin/env python
#
# Lara Maia <dev@lara.monster> 2015 ~ 2022
#
# The Steam Tools NG is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of
# the License, or (at your option) any later version.
#
# The Steam Tools NG is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see http://www.gnu.org/licenses/.
#
import asyncio
import logging
import os
import sys
from typing import Optional, Union, List, Tuple, Any

from .. import i18n, core

log = logging.getLogger(__name__)
_ = i18n.get_translation


def safe_input(
        msg: str,
        default_response: Optional[bool] = None,
        custom_choices: Optional[List[str]] = None,
) -> Union[bool, str]:
    if default_response and custom_choices:
        raise AttributeError("You can not use both default_response and custom_choices")

    if default_response is True:
        options = _('[Y/n]')
    elif default_response is False:
        options = _('[y/N]')
    elif custom_choices:
        options = f"Your choice [{'/'.join(custom_choices)}]"
    else:
        options = ''

    while True:
        try:
            user_input = input(f'{msg} {options}: ')

            if custom_choices:
                if not user_input:
                    raise ValueError(_('Invalid response from user'))

                if user_input.lower() in custom_choices:
                    return user_input.lower()

                raise ValueError(_('{} is not an accepted value').format(user_input))

            if default_response is None:
                if len(user_input) > 2:
                    return user_input

                raise ValueError(_('Invalid response from user'))

            if not user_input:
                return default_response

            if user_input.lower() == _('y'):
                return True

            if user_input.lower() == _('n'):
                return False

            raise ValueError(_('{} is not an accepted value').format(user_input))
        except ValueError as exception:
            log.error(exception.args[0])
            log.error(_('Please, try again.'))


def set_console(
        module_data: Optional[core.utils.ModuleData] = None,
        *,
        display: str = '',
        status: str = '',
        info: str = '',
        error: str = '',
        level: Tuple[int, int] = (0, 0),
) -> None:
    for std in (sys.stdout, sys.stderr):
        print(' ' * (os.get_terminal_size().columns - 1), end='\r', file=std)

    if not module_data:
        module_data = core.utils.ModuleData(display, status, info, error, level)

    if module_data.error:
        log.error(module_data.error)
        return

    if module_data.status:
        print(module_data.status, end=' ')

    if module_data.display:
        print(module_data.display, end=' ')

    if module_data.level:
        progress = module_data.level[0] + 1
        total = module_data.level[1]
        bar_size = 20

        if total > 0:
            total = int(progress * bar_size / total)
        else:
            total = bar_size

        print(f"┌{'█' * total:{bar_size}}┐", end=' ')

    if module_data.info:
        print(module_data.info, sep=' ', end=' ')

    print('', end='\r')


def safe_task_callback(task: asyncio.Task[Any]) -> None:
    if task.cancelled():
        log.debug(_("%s has been stopped due user request"), task.get_coro())
        return

    exception = task.exception()

    if exception and not isinstance(exception, asyncio.CancelledError):
        stack = task.get_stack()

        for frame in stack:
            log.critical("%s at %s", type(exception).__name__, frame)

        log.critical("Fatal Error: %s", str(exception))
        loop = asyncio.get_running_loop()
        loop.stop()
        sys.exit(1)



#!/usr/bin/env python
#
# Lara Maia <dev@lara.monster> 2015 ~ 2022
#
# The Steam Tools NG is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of
# the License, or (at your option) any later version.
#
# The Steam Tools NG is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see http://www.gnu.org/licenses/.
#
import asyncio
import logging
import random
from typing import AsyncGenerator

import aiohttp

from stlib import plugins, login
from . import utils
from .. import i18n, config

_ = i18n.get_translation
log = logging.getLogger(__name__)


async def main() -> AsyncGenerator[utils.ModuleData, None]:
    yield utils.ModuleData(status=_("Loading"))

    if plugins.has_plugin("steamgifts"):
        steamgifts = plugins.get_plugin("steamgifts")
        steamgifts_session = steamgifts.Main.get_session(0)
    else:
        raise ImportError(_("Unable to find Steamgifts plugin."))

    wait_min = config.parser.getint("steamgifts", "wait_min")
    wait_max = config.parser.getint("steamgifts", "wait_max")
    type_ = config.parser.get("steamgifts", "giveaway_type")
    pinned = config.parser.get("steamgifts", "developer_giveaways")
    sort = config.parser.get("steamgifts", "sort")
    reverse = config.parser.getboolean("steamgifts", "reverse_sorting")

    try:
        await steamgifts_session.do_login()
    except aiohttp.ClientError:
        yield utils.ModuleData(error=_("Check your connection. (server down?)"), info=_("Waiting Changes"))
        await asyncio.sleep(15)
        return
    except steamgifts.TooFast:
        yield utils.ModuleData(error=_("Unable to login. Trying again in 15 seconds"))
        await asyncio.sleep(15)
        return
    except steamgifts.UserSuspended:
        yield utils.ModuleData(error=_("User is suspended."))
        await asyncio.sleep(18000)
        return
    except steamgifts.PrivateProfile:
        yield utils.ModuleData(error=_("Your profile must be public to use steamgifts."), info=_("Waiting Changes"))
        await asyncio.sleep(30)
        return
    except login.LoginError:
        yield utils.ModuleData(error=_("User is not logged in. Trying again in 30 seconds"))
        await asyncio.sleep(30)
        return

    try:
        await steamgifts_session.configure()
    except aiohttp.ClientError:
        yield utils.ModuleData(error=_("Check your connection. (server down?)"))
        await asyncio.sleep(15)
        return
    except steamgifts.ConfigureError:
        yield utils.ModuleData(error=_("Unable to configure steamgifts."))
        await asyncio.sleep(20)
        return

    try:
        giveaways = await steamgifts_session.get_giveaways(type_, pinned_giveaways=pinned)
    except aiohttp.ClientError:
        yield utils.ModuleData(error=_("Check your connection. (server down?)"))
        await asyncio.sleep(15)
        return

    joined = False

    if giveaways:
        if sort:
            giveaways = sorted(
                giveaways,
                key=lambda giveaway_: getattr(giveaway_, sort),  # type: ignore
                reverse=reverse,
            )
    else:
        yield utils.ModuleData(status=_("No giveaways to join."))
        joined = True

    for index, giveaway in enumerate(giveaways):
        yield utils.ModuleData(level=(index, len(giveaway)))

        max_ban_wait = random.randint(5, 15)
        for past_time in range(max_ban_wait):
            try:
                yield utils.ModuleData(level=(past_time, max_ban_wait))
            except KeyError:
                yield utils.ModuleData(level=(0, 0))

            await asyncio.sleep(1)

        try:
            if await steamgifts_session.join(giveaway):
                yield utils.ModuleData(
                    display=giveaway.id,
                    status=f"{_('Joined')} {giveaway.name} "
                           f"(C:{giveaway.copies} P:{giveaway.points} L:{giveaway.level})",
                )
                joined = True
            else:
                yield utils.ModuleData(display=giveaway.id, error=_("Unable to join {}."))
                await asyncio.sleep(5)
                continue
        except aiohttp.ClientError:
            yield utils.ModuleData(error=_("Check your connection. (server down?)"))
            await asyncio.sleep(15)
            joined = False
            break
        except steamgifts.NoGiveawaysError:
            yield utils.ModuleData(error=_("No giveaways available to join."))
            await asyncio.sleep(15)
            continue
        except steamgifts.GiveawayEndedError:
            yield utils.ModuleData(error=_("Giveaway is already ended."))
            await asyncio.sleep(5)
            continue
        except login.LoginError:
            yield utils.ModuleData(error=_("Login is lost. Trying to relogin."))
            await asyncio.sleep(5)
            joined = False
            break
        except steamgifts.NoLevelError:
            yield utils.ModuleData(error=_("User don't have required level to join."))
            await asyncio.sleep(5)
            continue
        except steamgifts.NoPointsError:
            yield utils.ModuleData(error=_("User don't have required points to join."))
            await asyncio.sleep(5)

            if steamgifts_session.user_info.points <= 2:
                break

            continue

    if not joined:
        await asyncio.sleep(10)
        return

    wait_offset = random.randint(wait_min, wait_max)
    log.debug(_("Setting wait_offset from steamgifts to %s."), wait_offset)
    for past_time in range(wait_offset):
        yield utils.ModuleData(
            info=_("Waiting more {} minutes.").format(round((wait_offset - past_time) / 60)),
            level=(past_time, wait_offset),
        )

        await asyncio.sleep(1)



#!/usr/bin/env python
#
# Lara Maia <dev@lara.monster> 2015 ~ 2022
#
# The Steam Tools NG is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of
# the License, or (at your option) any later version.
#
# The Steam Tools NG is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see http://www.gnu.org/licenses/.
#
import asyncio
import contextlib
import logging
from subprocess import call
from typing import Union, Optional, Tuple, Any, List

from gi.repository import Gio, Gtk

import stlib
from stlib import login, universe
from . import confirmation, utils, coupon
from .. import config, i18n, core

_ = i18n.get_translation
log = logging.getLogger(__name__)

if stlib.steamworks_available:
    from stlib import client


# noinspection PyUnusedLocal
class Main(Gtk.ApplicationWindow):
    def __init__(self, application: Gtk.Application, title: str) -> None:
        super().__init__(application=application, title=title)
        self.application = application
        header_bar = Gtk.HeaderBar()

        menu = Gio.Menu()
        menu.append(_("Settings"), "app.settings")
        menu.append(_("About"), "app.about")
        menu.append(_("Exit"), "app.exit")

        menu_button = Gtk.MenuButton()
        menu_button.set_icon_name("open-menu")
        menu_button.set_menu_model(menu)
        header_bar.pack_end(menu_button)

        self.set_default_size(650, 10)
        self.set_resizable(False)

        if config.parser.getboolean("general", "show_close_button"):
            self.set_deletable(True)
        else:
            self.set_deletable(False)

        self.set_titlebar(header_bar)
        self.set_title('Steam Tools NG')

        main_grid = Gtk.Grid()
        main_grid.set_margin_start(10)
        main_grid.set_margin_end(10)
        main_grid.set_margin_top(10)
        main_grid.set_margin_bottom(10)
        self.set_child(main_grid)

        self.user_info_label = Gtk.Label()
        self.user_info_label.set_halign(Gtk.Align.END)
        header_bar.pack_start(self.user_info_label)

        stack = Gtk.Stack()
        stack.set_hhomogeneous(True)
        main_grid.attach(stack, 1, 2, 1, 1)

        switcher = Gtk.StackSwitcher()
        switcher.set_stack(stack)
        main_grid.attach(switcher, 1, 1, 1, 1)

        general_section = utils.Section("general", _("General"))
        general_section.stackup_section(stack)

        coupons_section = utils.Section("coupons", _("Free Coupons"))
        coupons_section.stackup_section(stack)

        self.status_grid = Gtk.Grid()
        self.status_grid.set_row_spacing(10)
        self.status_grid.set_column_spacing(10)
        self.status_grid.set_column_homogeneous(True)
        general_section.grid.attach(self.status_grid, 0, 0, 4, 1)

        # grid managed by plugin switch
        self.steamtrades_status = utils.Status(5, config.plugins['steamtrades'])
        self.steamgifts_status = utils.Status(5, config.plugins['steamgifts'])
        self.steamguard_status = utils.Status(4, config.plugins['steamguard'])
        self.cardfarming_status = utils.Status(6, config.plugins['cardfarming'])

        self.confirmations_grid = Gtk.Grid()
        self.confirmations_grid.set_size_request(655, 250)
        self.confirmations_grid.set_row_spacing(10)
        general_section.grid.attach(self.confirmations_grid, 0, 1, 4, 1)

        self.confirmation_tree = utils.SimpleTextTree(
            _('confid'), _('creatorid'), _('key'), _('give'), _('to'), _('receive'),
            overlay_scrolling=False,
        )

        self.confirmations_grid.attach(self.confirmation_tree, 0, 0, 4, 1)

        for index, column in enumerate(self.confirmation_tree.view.get_columns()):
            if index in (0, 1, 2):
                column.set_visible(False)

            if index == 4:
                column.set_fixed_width(140)
            else:
                column.set_fixed_width(220)

        self.confirmation_tree.view.set_has_tooltip(True)
        self.confirmation_tree.view.connect('query-tooltip', self.on_query_confirmations_tooltip)

        confirmation_tree_selection = self.confirmation_tree.view.get_selection()
        confirmation_tree_selection.connect("changed", self.on_tree_selection_changed)

        accept_button = Gtk.Button()
        accept_button.set_margin_start(5)
        accept_button.set_margin_end(5)
        accept_button.set_label(_('Accept selected'))
        accept_button.connect('clicked', self.on_validate_confirmations, "allow", confirmation_tree_selection)
        self.confirmations_grid.attach(accept_button, 0, 1, 1, 1)

        cancel_button = Gtk.Button()
        cancel_button.set_margin_start(5)
        cancel_button.set_margin_end(5)
        cancel_button.set_label(_('Cancel selected'))
        cancel_button.connect('clicked', self.on_validate_confirmations, "cancel", confirmation_tree_selection)
        self.confirmations_grid.attach(cancel_button, 1, 1, 1, 1)

        accept_all_button = Gtk.Button()
        accept_all_button.set_margin_start(5)
        accept_all_button.set_margin_end(5)
        accept_all_button.set_label(_('Accept all'))
        accept_all_button.connect('clicked', self.on_validate_confirmations, "allow", self.confirmation_tree.store)
        self.confirmations_grid.attach(accept_all_button, 2, 1, 1, 1)

        cancel_all_button = Gtk.Button()
        cancel_all_button.set_margin_start(5)
        cancel_all_button.set_margin_end(5)
        cancel_all_button.set_label(_('Cancel all'))
        cancel_all_button.connect('clicked', self.on_validate_confirmations, "cancel", self.confirmation_tree.store)
        self.confirmations_grid.attach(cancel_all_button, 3, 1, 1, 1)

        self.coupon_grid = Gtk.Grid()
        self.coupon_grid.set_size_request(655, 300)
        self.coupon_grid.set_row_spacing(10)
        coupons_section.grid.attach(self.coupon_grid, 0, 0, 1, 1)

        self.coupon_tree = utils.SimpleTextTree(
            _('price'), _('name'), 'link', 'botid', 'token', 'assetid',
            overlay_scrolling=False,
            model=Gtk.ListStore,
        )

        self.coupon_tree.store.set_sort_column_id(0, Gtk.SortType.ASCENDING)
        self.coupon_tree.store.set_sort_func(0, self.coupon_sorting)
        self.coupon_grid.attach(self.coupon_tree, 0, 0, 4, 2)

        for index, column in enumerate(self.coupon_tree.view.get_columns()):
            if index in (2, 3, 4, 5):
                column.set_visible(False)

        self.coupon_tree.view.connect('row-activated', self.on_coupon_double_clicked)

        coupon_tree_selection = self.coupon_tree.view.get_selection()
        coupon_tree_selection.connect("changed", self.on_tree_selection_changed)

        self.coupon_progress = Gtk.LevelBar()
        self.coupon_grid.attach(self.coupon_progress, 0, 3, 4, 1)

        get_coupon_button = Gtk.Button()
        get_coupon_button.set_margin_start(5)
        get_coupon_button.set_margin_end(5)
        get_coupon_button.set_label(_('Get selected coupon'))
        get_coupon_button.connect('clicked', self.on_coupon_action, coupon_tree_selection)
        self.coupon_grid.attach(get_coupon_button, 2, 4, 1, 1)

        give_coupon_button = Gtk.Button()
        give_coupon_button.set_margin_start(5)
        give_coupon_button.set_margin_end(5)
        give_coupon_button.set_label(_('Giveaway your coupons'))
        give_coupon_button.connect('clicked', self.on_coupon_action)
        self.coupon_grid.attach(give_coupon_button, 3, 4, 1, 1)

        fetch_coupons_button = Gtk.Button()
        fetch_coupons_button.set_margin_start(5)
        fetch_coupons_button.set_margin_end(5)
        fetch_coupons_button.set_label(_('Fetch coupons'))
        fetch_coupons_button.connect('clicked', self.on_fetch_coupons)
        self.coupon_grid.attach(fetch_coupons_button, 0, 4, 1, 1)

        self.fetch_coupon_event = asyncio.Event()

        stop_fetching_coupons_button = Gtk.Button()
        stop_fetching_coupons_button.set_margin_start(5)
        stop_fetching_coupons_button.set_margin_end(5)
        stop_fetching_coupons_button.set_label(_('Stop fetching coupons'))
        stop_fetching_coupons_button.connect('clicked', self.on_stop_fetching_coupons)
        self.coupon_grid.attach(stop_fetching_coupons_button, 1, 4, 1, 1)

        self.statusbar = utils.StatusBar()
        main_grid.attach(self.statusbar, 1, 3, 1, 1)

        self.connect("destroy", self.application.on_exit_activate)
        self.connect("close-request", self.application.on_exit_activate)

        loop = asyncio.get_event_loop()

        plugin_switch_task = loop.create_task(self.plugin_switch())
        plugin_switch_task.add_done_callback(utils.safe_task_callback)

        user_info_task = loop.create_task(self.user_info())
        user_info_task.add_done_callback(utils.safe_task_callback)

    async def user_info(self) -> None:
        while self.get_realized():
            theme = config.parser.get('general', 'theme')
            account_name = config.parser.get('login', 'account_name')
            steamid_raw = config.parser.get('login', 'steamid')
            steamid = universe.generate_steamid(steamid_raw)
            login_session = None

            with contextlib.suppress(IndexError):
                login_session = login.Login.get_session(0)

            if not login_session or not await login_session.is_logged_in(steamid):
                self.application.main_window.user_info_label.set_markup(
                    utils.markup(
                        _('You are not logged in'),
                        color='darkred' if theme == 'light' else 'red',
                        size='small',
                    )
                )

                await asyncio.sleep(5)
                continue

            self.application.main_window.user_info_label.set_markup(
                utils.markup(
                    _('You are logged in as:\n'),
                    color='darkgreen' if theme == 'light' else 'green',
                    size='small',
                ) +
                utils.markup(
                    account_name,
                    color='darkblue' if theme == 'light' else 'blue',
                    size='small',
                ) +
                utils.markup(
                    f" {self.application.steamid.id3_string}",
                    color='grey',
                    size='small',
                )
            )

            await asyncio.sleep(30)

    async def plugin_switch(self) -> None:
        plugins_enabled: List[str] = []
        plugins_status: List[utils.Status] = []

        for plugin_name in config.plugins.keys():
            if plugin_name in ["confirmations", "coupons"]:
                continue

            plugins_status.append(getattr(self, f'{plugin_name}_status'))

        while self.get_realized():
            plugins: List[str] = []

            for plugin_name in config.plugins.keys():
                enabled = config.parser.getboolean(plugin_name, "enable")

                if plugin_name == "confirmations":
                    if enabled:
                        self.confirmations_grid.show()
                    else:
                        self.confirmations_grid.hide()

                    continue

                if plugin_name == "coupons":
                    if enabled:
                        self.coupon_grid.show()
                    else:
                        self.coupon_grid.hide()

                    continue

                if enabled:
                    plugins.append(plugin_name)

            if plugins == plugins_enabled:
                self.status_grid.show()
                await asyncio.sleep(1)
                continue

            plugins_enabled = plugins

            for status in plugins_status:
                self.status_grid.remove(status)

            for index, plugin_name in enumerate(plugins_enabled):
                status = getattr(self, f'{plugin_name}_status')

                if index == 0:
                    if len(plugins_enabled) >= 2:
                        self.status_grid.attach(status, 0, 0, 1, 1)
                    else:
                        self.status_grid.attach(status, 0, 0, 2, 1)

                if index == 1 and len(plugins_enabled) >= 2:
                    self.status_grid.attach(status, 1, 0, 1, 1)

                if index == 2:
                    if len(plugins_enabled) == 3:
                        self.status_grid.attach(status, 0, 1, 2, 1)
                    else:
                        self.status_grid.attach(status, 0, 1, 1, 1)

                if index == 3 and len(plugins_enabled) == 4:
                    self.status_grid.attach(status, 1, 1, 1, 1)

                status.show()

    @staticmethod
    def on_query_confirmations_tooltip(
            tree_view: Gtk.TreeView,
            x_coord: int,
            y_coord: int,
            keyboard_tip: bool,
            tooltip: Gtk.Tooltip,
    ) -> bool:
        context = tree_view.get_tooltip_context(x_coord, y_coord, keyboard_tip)

        if context[0]:
            if context.model.iter_depth(context.iter) != 0:
                return False

            tooltip.set_text(
                f'ConfID:{context.model.get_value(context.iter, 0)}\n'
                f'CreatorID{context.model.get_value(context.iter, 1)}\n'
                f'Key:{context.model.get_value(context.iter, 2)}'
            )

            return True

        return False

    def on_fetch_coupons(self, button: Gtk.Button) -> None:
        self.fetch_coupon_event.set()

    def on_stop_fetching_coupons(self, button: Gtk.Button) -> None:
        self.fetch_coupon_event.clear()

    def on_coupon_action(self, button: Gtk.Button, model: Union[Gtk.TreeModel, Gtk.TreeSelection] = None) -> None:
        if model:
            coupon_dialog = coupon.CouponDialog(self, self.application, *model.get_selected())
        else:
            coupon_dialog = coupon.CouponDialog(self, self.application)

        coupon_dialog.show()

    def on_validate_confirmations(
            self,
            button: Gtk.Button,
            action: str,
            model: Union[Gtk.TreeModel, Gtk.TreeSelection]) -> None:
        if isinstance(model, Gtk.TreeModel):
            finalize_dialog = confirmation.FinalizeDialog(
                self,
                self.application,
                action,
                model,
                False
            )
        else:
            finalize_dialog = confirmation.FinalizeDialog(
                self,
                self.application,
                action,
                *model.get_selected()
            )

        finalize_dialog.show()

    @staticmethod
    def on_coupon_double_clicked(view: Gtk.TreeView, path: Gtk.TreePath, column: Gtk.TreeViewColumn) -> None:
        model = view.get_model()
        url = model[path][2]
        steam_running = False

        if stlib.steamworks_available:
            with contextlib.suppress(ProcessLookupError):
                with client.SteamGameServer() as server:
                    steam_running = True

        if steam_running:
            url = f"steam://openurl/{url}"

        call(f'{config.file_manager} "{url}"')

    @staticmethod
    def on_tree_selection_changed(selection: Gtk.TreeSelection) -> None:
        model, iter_ = selection.get_selected()

        if iter_:
            parent = model.iter_parent(iter_)

            if parent:
                selection.select_iter(parent)

    @staticmethod
    def coupon_sorting(model: Gtk.TreeModel, iter1: Gtk.TreeIter, iter2: Gtk.TreeIter, user_data: Any) -> Any:
        column, _ = model.get_sort_column_id()
        price1 = model.get_value(iter1, column)
        price2 = model.get_value(iter2, column)

        if float(price1) < float(price2):
            return -1

        if price1 == price2:
            return 0

        return 1

    def set_status(
            self,
            module: str,
            module_data: Optional[core.utils.ModuleData] = None,
            *,
            display: str = '',
            status: str = '',
            info: str = '',
            error: str = '',
            level: Tuple[int, int] = (0, 0),
    ) -> None:
        _status = getattr(self, f'{module}_status')

        if not module_data:
            module_data = core.utils.ModuleData(display, status, info, error, level)

        if module_data.display:
            # log.debug(f"display data: {module_data.display}")
            _status.set_display(module_data.display)
        else:
            _status.unset_display()

        if module_data.status:
            # log.debug(f"status data: {module_data.status}")
            _status.set_status(module_data.status)

        if module_data.info:
            # log.debug(f"info data: {module_data.info}")
            _status.set_info(module_data.info)

        if module_data.error:
            log.error(module_data.error)
            _status.set_error(module_data.error)

        if module_data.level:
            _status.set_level(*module_data.level)

    def get_play_event(self, module: str) -> asyncio.Event:
        _status = getattr(self, f'{module}_status')
        assert isinstance(_status, utils.Status)
        return _status.play_event



#!/usr/bin/env python
#
# Lara Maia <dev@lara.monster> 2015 ~ 2022
#
# The Steam Tools NG is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of
# the License, or (at your option) any later version.
#
# The Steam Tools NG is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see http://www.gnu.org/licenses/.
#
from typing import Type

import asyncio
import logging
from collections import OrderedDict
from subprocess import call

from gi.repository import Gtk, Pango

import stlib
from stlib import plugins

from . import utils, advanced, authenticator, login
from .. import config, i18n

log = logging.getLogger(__name__)
_ = i18n.get_translation


# noinspection PyUnusedLocal
class SettingsDialog(Gtk.Dialog):
    def __init__(
            self,
            parent_window: Gtk.Window,
            application: Gtk.Application,
    ) -> None:
        super().__init__(use_header_bar=True)
        self.parent_window = parent_window
        self.application = application

        self.set_default_size(300, 150)
        self.set_title(_('Settings'))
        self.set_transient_for(parent_window)
        self.set_modal(True)
        self.set_destroy_with_parent(True)
        self.set_resizable(False)

        self.gtk_settings_class = Gtk.Settings.get_default()

        content_area = self.get_content_area()
        content_grid = Gtk.Grid()
        content_grid.set_row_spacing(10)
        content_grid.set_column_spacing(10)
        content_area.append(content_grid)

        stack = Gtk.Stack()
        stack.set_margin_end(10)
        content_grid.attach(stack, 1, 0, 1, 1)

        sidebar = Gtk.StackSidebar()
        sidebar.set_stack(stack)
        content_grid.attach(sidebar, 0, 0, 1, 1)

        general_section = utils.Section("general", _("General"))

        config_button = Gtk.Button()
        config_button.set_label(_("Config File Directory"))
        config_button.set_name("config_button")
        config_button.set_hexpand(True)
        config_button.connect("clicked", self.on_config_button_clicked)

        if config.parser.get("logger", "log_directory") == str(config.config_file_directory):
            config_button.set_label(_("Config / Log file Directory"))
            general_section.grid.attach(config_button, 0, 1, 2, 1)
        else:
            log_button = Gtk.Button()
            log_button.set_label(_("Log File Directory"))
            log_button.set_name("log_button")
            log_button.connect("clicked", self.on_log_button_clicked)
            general_section.grid.attach(config_button, 0, 1, 1, 1)
            general_section.grid.attach(log_button, 1, 1, 1, 1)

        theme = general_section.new_item("theme", _("Theme:"), Gtk.ComboBoxText, 0, 3, items=config.gtk_themes)
        theme.connect('changed', self.on_theme_changed)

        show_close_button = general_section.new_item("show_close_button", _("Show close button:"), Gtk.CheckButton, 0, 4)
        show_close_button.connect('toggled', self.on_show_close_button_toggled)

        language_item = general_section.new_item(
            "language", _("Language"), Gtk.ComboBoxText, 0, 5, items=config.translations
        )
        language_item.connect("changed", self.update_language)

        login_section = utils.Section("login", _("Login"))

        account_name = login_section.new_item('account_name', _("Username:"), Gtk.Entry, 0, 0)
        account_name.connect('changed', on_setting_changed)

        login_button = Gtk.Button()
        login_button.set_label(_("Login with another account"))
        login_button.set_name("login_button")
        login_button.connect('clicked', self.on_login_button_clicked)
        login_section.grid.attach(login_button, 0, 1, 2, 1)

        new_authenticator_button = Gtk.Button()
        new_authenticator_button.set_label(_("Use STNG as your Steam Authenticator"))
        new_authenticator_button.set_name("new_authenticator_button")
        new_authenticator_button.connect("clicked", self.on_new_authenticator_clicked)
        login_section.grid.attach(new_authenticator_button, 0, 2, 2, 1)

        reset_password_button = Gtk.Button()
        reset_password_button.set_label(_("Remove Saved Password"))
        reset_password_button.set_name("reset_password_button")
        reset_password_button.connect("clicked", self.on_reset_password_clicked)
        login_section.grid.attach(reset_password_button, 0, 3, 2, 1)

        advanced_button = Gtk.ToggleButton()
        advanced_button.set_label(_("Advanced"))
        advanced_button.set_name("advanced_button")
        advanced_button.connect("toggled", self.on_advanced_button_toggled)
        login_section.grid.attach(advanced_button, 0, 4, 2, 1)

        confirmations_section = utils.Section('confirmations', _('Confirmations'))

        confirmations_enable = confirmations_section.new_item("enable", _("Enable:"), Gtk.CheckButton, 0, 0)
        confirmations_enable.connect('toggled', on_setting_toggled)

        if not config.parser.get("login", "identity_secret"):
            confirmations_section.set_sensitive(False)
            confirmations_enable.set_active(False)
            _confirmations_disabled = Gtk.Label()
            _confirmations_disabled.set_justify(Gtk.Justification.CENTER)
            _confirmations_disabled.set_halign(Gtk.Align.CENTER)

            _message = _(
                "confirmations module has been disabled because you have\n"
                "logged in but no identity secret is found. To enable it again,\n"
                "go to login -> advanced and add a valid identity secret\n"
                "or use STNG as your Steam Authenticator\n"
            )

            _confirmations_disabled.set_markup(utils.markup(_message, color="hotpink", background="black"))
            confirmations_section.grid.attach(_confirmations_disabled, 0, 1, 4, 4)

        steamguard_section = utils.Section('steamguard', _('SteamGuard'))

        steamguard_enable = steamguard_section.new_item("enable", _("Enable:"), Gtk.CheckButton, 0, 0)
        steamguard_enable.connect('toggled', on_setting_toggled)

        if not config.parser.get("login", "shared_secret"):
            steamguard_section.set_sensitive(False)
            steamguard_enable.set_active(False)
            _steamguard_disabled = Gtk.Label()
            _steamguard_disabled.set_justify(Gtk.Justification.CENTER)
            _steamguard_disabled.set_halign(Gtk.Align.CENTER)

            _message = _(
                "steamguard module has been disabled because you have\n"
                "logged in but no shared secret is found. To enable it again,\n"
                "go to login -> advanced and add a valid shared secret\n"
                "or use STNG as your Steam Authenticator\n"
            )

            _steamguard_disabled.set_markup(utils.markup(_message, color="hotpink", background="black"))
            steamguard_section.grid.attach(_steamguard_disabled, 0, 1, 4, 4)

        steamtrades_section = utils.Section('steamtrades', _('Steamtrades'))

        steamtrades_enable = steamtrades_section.new_item("enable", _("Enable:"), Gtk.CheckButton, 0, 0)
        steamtrades_enable.connect('toggled', on_setting_toggled)

        trade_ids = steamtrades_section.new_item("trade_ids", _("Trade IDs:"), Gtk.Entry, 0, 1)
        trade_ids.set_placeholder_text('12345, asdfg, ...')
        trade_ids.connect("changed", on_setting_changed)

        wait_min = steamtrades_section.new_item("wait_min", _("Wait MIN:"), Gtk.Entry, 0, 2)
        wait_min.connect("changed", on_digit_only_setting_changed)

        wait_max = steamtrades_section.new_item("wait_max", _("Wait MAX:"), Gtk.Entry, 0, 3)
        wait_max.connect("changed", on_digit_only_setting_changed)

        if not plugins.has_plugin("steamtrades"):
            steamtrades_section.set_sensitive(False)
            steamtrades_enable.set_active(False)
            _steamtrades_disabled = Gtk.Label()
            _steamtrades_disabled.set_justify(Gtk.Justification.CENTER)
            _steamtrades_disabled.set_halign(Gtk.Align.CENTER)

            _message = _(
                "steamtrades module has been disabled because you don't\n"
                "have steamtrades plugin installed. To enable it again,\n"
                "install the steamtrades plugin.\n"
            )

            _steamtrades_disabled.set_markup(utils.markup(_message, color="hotpink", background="black"))
            steamtrades_section.grid.attach(_steamtrades_disabled, 0, 1, 4, 4)

        steamgifts_section = utils.Section("steamgifts", _("Steamgifts"))

        steamgifts_enable = steamgifts_section.new_item("enable", _("Enable:"), Gtk.CheckButton, 0, 0)
        steamgifts_enable.connect('toggled', on_setting_toggled)

        giveaway_type = steamgifts_section.new_item(
            "giveaway_type",
            _("Giveaway Type:"),
            Gtk.ComboBoxText,
            0, 1,
            items=config.giveaway_types,
        )
        giveaway_type.connect("changed", on_combo_setting_changed, config.giveaway_types)

        sort_giveaways = steamgifts_section.new_item(
            "sort",
            _("Sort Giveaways:"),
            Gtk.ComboBoxText,
            0, 2,
            items=config.giveaway_sort_types,
        )
        sort_giveaways.connect("changed", on_combo_setting_changed, config.giveaway_sort_types)

        reverse_sorting = steamgifts_section.new_item("reverse_sorting", _("Reverse Sorting:"), Gtk.CheckButton, 0, 3)
        reverse_sorting.connect("toggled", on_setting_toggled)

        developer_giveaways = steamgifts_section.new_item(
            "developer_giveaways",
            _("Developer Giveaways"),
            Gtk.CheckButton,
            0, 4,
        )
        developer_giveaways.connect("toggled", on_setting_toggled)

        wait_min = steamgifts_section.new_item("wait_min", _("Wait MIN:"), Gtk.Entry, 0, 5)
        wait_min.connect("changed", on_digit_only_setting_changed)

        wait_max = steamgifts_section.new_item("wait_max", _("Wait MAX:"), Gtk.Entry, 0, 6)
        wait_max.connect("changed", on_digit_only_setting_changed)

        if not plugins.has_plugin("steamgifts"):
            steamgifts_section.set_sensitive(False)
            steamgifts_enable.set_active(False)
            _steamgifts_disabled = Gtk.Label()
            _steamgifts_disabled.set_justify(Gtk.Justification.CENTER)
            _steamgifts_disabled.set_halign(Gtk.Align.CENTER)

            _message = _(
                "steamgifts module has been disabled because you don't\n"
                "have steamgifts plugin installed. To enable it again,\n"
                "install the steamgifts plugin.\n"
            )

            _steamgifts_disabled.set_markup(utils.markup(_message, color="hotpink", background="black"))
            steamgifts_section.grid.attach(_steamgifts_disabled, 0, 1, 4, 4)

        cardfarming_section = utils.Section("cardfarming", _("Cardfarming"))

        cardfarming_enable = cardfarming_section.new_item("enable", _("Enable:"), Gtk.CheckButton, 0, 0)
        cardfarming_enable.connect("toggled", on_setting_toggled)

        first_wait = cardfarming_section.new_item("first_wait", _("First Wait:"), Gtk.Entry, 0, 1)
        first_wait.connect("changed", on_digit_only_setting_changed)

        default_wait = cardfarming_section.new_item("default_wait", _("Default Wait:"), Gtk.Entry, 0, 2)
        default_wait.connect("changed", on_digit_only_setting_changed)

        min_wait = cardfarming_section.new_item("min_wait", _("MIN Wait:"), Gtk.Entry, 0, 3)
        min_wait.connect("changed", on_digit_only_setting_changed)

        max_concurrency = cardfarming_section.new_item("max_concurrency", _("Max concurrency:"), Gtk.Entry, 0, 4)
        max_concurrency.connect("changed", on_digit_only_setting_changed)

        reverse_sorting = cardfarming_section.new_item("reverse_sorting", _("More cards first:"), Gtk.CheckButton, 0, 5)
        reverse_sorting.connect("toggled", on_setting_toggled)

        cardfarming_invisible = cardfarming_section.new_item("invisible", _("Invisible:"), Gtk.CheckButton, 0, 6)
        cardfarming_invisible.connect("toggled", on_setting_toggled)

        if not stlib.steamworks_available:
            cardfarming_section.set_sensitive(False)
            cardfarming_enable.set_active(False)
            _cardfarming_disabled = Gtk.Label()
            _cardfarming_disabled.set_justify(Gtk.Justification.CENTER)
            _cardfarming_disabled.set_halign(Gtk.Align.CENTER)

            _message = _(
                "cardfarming module has been disabled because you have\n"
                "a stlib built without SteamWorks support. To enable it again,\n"
                "reinstall stlib with SteamWorks support\n"
            )

            _cardfarming_disabled.set_markup(utils.markup(_message, color="hotpink", background="black"))
            cardfarming_section.grid.attach(_cardfarming_disabled, 0, 1, 4, 4)

        logger_section = utils.Section("logger", _('Logger'))

        log_level_item = logger_section.new_item("log_level", _("Level:"), Gtk.ComboBoxText, 0, 0, items=config.log_levels)

        log_console_level_item = logger_section.new_item(
            "log_console_level",
            _("Console level:"),
            Gtk.ComboBoxText,
            0, 1,
            items=config.log_levels,
        )

        log_level_item.connect("changed", on_combo_setting_changed, config.log_levels)
        log_console_level_item.connect("changed", on_combo_setting_changed, config.log_levels)

        log_color = logger_section.new_item("log_color", _("Log color:"), Gtk.CheckButton, 0, 2)
        log_color.connect("toggled", on_setting_toggled)

        coupons_section = utils.Section("coupons", _("Coupons"))

        coupon_enable = coupons_section.new_item("enable", _("Enable:"), Gtk.CheckButton, 0, 0)
        coupon_enable.connect("toggled", on_setting_toggled)

        coupon_botids = coupons_section.new_item("botids", _("BotIDs:"), Gtk.Entry, 0, 1)
        coupon_botids.set_placeholder_text('12345, asdfg, ...')
        coupon_botids.connect("changed", on_setting_changed)

        coupon_tokens = coupons_section.new_item("tokens", _("Tokens:"), Gtk.Entry, 0, 2)
        coupon_tokens.set_placeholder_text('12345, asdfg, ...')
        coupon_tokens.connect("changed", on_setting_changed)

        coupon_botid_to_donate = coupons_section.new_item("botid_to_donate", _("BotID To Donate:"), Gtk.Entry, 0, 3)
        coupon_botid_to_donate.connect("changed", on_digit_only_setting_changed)

        coupon_token_to_donate = coupons_section.new_item("token_to_donate", _("Token To Donate:"), Gtk.Entry, 0, 4)
        coupon_token_to_donate.connect("changed", on_setting_changed)

        self.connect('response', lambda dialog, response_id: self.destroy())

        for section in [
            general_section,
            login_section,
            logger_section,
            steamtrades_section,
            cardfarming_section,
            steamgifts_section,
            coupons_section,
            steamguard_section,
            confirmations_section,
        ]:
            section.stackup_section(stack)

    @staticmethod
    def on_log_button_clicked(button: Gtk.Button) -> None:
        call(f'{config.file_manager} {config.parser.get("logger", "log_directory")}')

    @staticmethod
    def on_config_button_clicked(button: Gtk.Button) -> None:
        call(f'{config.file_manager} {str(config.config_file_directory)}')

    def on_advanced_button_toggled(self, button: Gtk.Button) -> None:
        if button.get_active():
            advanced_settings = advanced.AdvancedSettingsDialog(self, self.application, button)
            advanced_settings.show()

    def on_show_close_button_toggled(self, button: Gtk.Button) -> None:
        current_value = button.get_active()

        if current_value:
            self.parent_window.set_deletable(True)
        else:
            self.parent_window.set_deletable(False)

        config.new('general', 'show_close_button', current_value)

    def on_theme_changed(self, combo: Gtk.ComboBoxText) -> None:
        theme = list(config.gtk_themes)[combo.get_active()]

        if theme == 'dark':
            self.gtk_settings_class.props.gtk_application_prefer_dark_theme = True
        else:
            self.gtk_settings_class.props.gtk_application_prefer_dark_theme = False

        config.new('general', 'theme', theme)

    def on_login_button_clicked(self, button: Gtk.Button) -> None:
        login_dialog = login.LoginDialog(self.parent_window, self.application)
        login_dialog.shared_secret_item.set_text('')
        login_dialog.identity_secret_item.set_text('')
        login_dialog.show()
        self.destroy()

    def on_new_authenticator_clicked(self, button: Gtk.Button) -> None:
        new_authenticator_dialog = authenticator.NewAuthenticatorDialog(self.parent_window, self.application)
        new_authenticator_dialog.show()
        self.destroy()

    def on_reset_password_clicked(self, button: Gtk.Button) -> None:
        login_dialog = login.LoginDialog(self.parent_window, self.application)
        login_dialog.status.info(_("Removing saved password..."))
        login_dialog.user_details_section.hide()
        login_dialog.advanced_login.hide()
        login_dialog.set_deletable(False)
        login_dialog.show()

        config.new("login", "password", "")

        def reset_password_callback() -> None:
            self.destroy()
            login_dialog.destroy()

        asyncio.get_event_loop().call_later(2, reset_password_callback)

    def update_language(self, combo: Gtk.ComboBoxText) -> None:
        language = list(config.translations)[combo.get_active()]
        config.new('general', 'language', language)
        refresh_widget_childrens(self)
        refresh_widget_childrens(self.parent_window)


def on_setting_toggled(checkbutton: Gtk.CheckButton) -> None:
    current_value = checkbutton.get_active()
    section = checkbutton.get_section_name()
    option = checkbutton.get_name()

    config.new(section, option, current_value)


def on_setting_changed(entry: Gtk.Entry) -> None:
    current_value = entry.get_text()
    section = entry.get_section_name()
    option = entry.get_name()

    config.new(section, option, current_value)


def on_digit_only_setting_changed(entry: Gtk.Entry) -> None:
    current_value = entry.get_text()
    section = entry.get_section_name()
    option = entry.get_name()

    if current_value.isdigit():
        config.new(section, option, int(current_value))
    else:
        entry.handler_block_by_func(on_digit_only_setting_changed)
        entry.set_text(utils.remove_letters(current_value))
        entry.handler_unblock_by_func(on_digit_only_setting_changed)


def on_combo_setting_changed(combo: Gtk.ComboBoxText, items: 'OrderedDict[str, str]') -> None:
    current_value = list(items)[combo.get_active()]
    section = combo.get_section_name()
    option = combo.get_name()

    config.new(section, option, current_value)


def refresh_widget_childrens(widget: Type[Gtk.Widget]) -> None:
    next_child = widget.get_first_child()

    while next_child:
        refresh_widget(next_child)
        next_child = next_child.get_next_sibling()


def refresh_widget(widget: Type[Gtk.Widget]) -> None:
    if isinstance(widget, Gtk.MenuButton):
        refresh_widget(widget.get_popover())
        return

    if isinstance(widget, Gtk.ComboBoxText):
        model = widget.get_model()

        for index, row in enumerate(model):
            combo_item_label = model.get_value(row.iter, 0)

            try:
                cached_text = i18n.cache[i18n.new_hash(combo_item_label)]
            except KeyError:
                log.debug("it's not an i18n string: %s", combo_item_label)
                return

            model.set_value(row.iter, 0, _(cached_text))

        log.debug('ComboBox refreshed: %s', widget)

    if isinstance(widget, Gtk.Label):
        try:
            cached_text = i18n.cache[i18n.new_hash(widget.get_text())]
        except KeyError:
            log.debug("it's not an i18n string: %s", widget.get_text())
            return

        if widget.get_use_markup():
            old_attributes = Pango.Layout.get_attributes(widget.get_layout())
            widget.set_text(_(cached_text))
            widget.set_attributes(old_attributes)
        else:
            widget.set_text(_(cached_text))

        log.debug('widget refreshed: %s', str(widget))
        return

    log.debug('widget not refresh: %s', str(widget))
    refresh_widget_childrens(widget)



#!/usr/bin/env python
#
# Lara Maia <dev@lara.monster> 2015 ~ 2022
#
# The Steam Tools NG is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of
# the License, or (at your option) any later version.
#
# The Steam Tools NG is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see http://www.gnu.org/licenses/.
#
import asyncio
from typing import AsyncGenerator

import aiohttp

from stlib import webapi, client, universe
from . import utils
from .. import i18n

_ = i18n.get_translation


async def main(steamid: universe.SteamId, game_id: int) -> AsyncGenerator[utils.ModuleData, None]:
    session = webapi.SteamWebAPI.get_session(0)

    try:
        game_list = await session.get_owned_games(steamid, appids_filter=[game_id])
        game_name = game_list[0].name
    except aiohttp.ClientError:
        yield utils.ModuleData(error=_("Check your connection. (server down?)"))
        await asyncio.sleep(15)
        return
    except ValueError:
        yield utils.ModuleData(error=_("Game {} doesn't exist").format(game_id))
        return

    yield utils.ModuleData(
        display=str(game_id),
        status=_("Loading {}").format(game_name),
    )

    start_time = 0

    try:
        with client.SteamAPIExecutor(game_id) as executor:
            while True:
                yield utils.ModuleData(
                    display=str(game_id),
                    info=_("Running for {} minutes.").format(round(start_time / 60)),
                    status=_("Running {}").format(game_name),
                    raw_data=executor,
                    action="check",
                )
                await asyncio.sleep(1)
                start_time += 1
    except ProcessLookupError:
        yield utils.ModuleData(error=_("Steam Client is not running."))



#!/usr/bin/env python
#
# Lara Maia <dev@lara.monster> 2015 ~ 2022
#
# The Steam Tools NG is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of
# the License, or (at your option) any later version.
#
# The Steam Tools NG is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see http://www.gnu.org/licenses/.
#
import asyncio
import binascii
from typing import AsyncGenerator

import aiohttp

from stlib import universe, webapi
from . import utils
from .. import i18n, config

try:
    from stlib import client
except ImportError as exception:
    client = None

_ = i18n.get_translation


@utils.time_offset_cache(ttl=3600)
def cached_server_time() -> int:
    if not client:
        raise ProcessLookupError

    with client.SteamGameServer() as server:
        real_time = server.get_server_real_time()
        assert isinstance(real_time, int)
        return real_time


async def main() -> AsyncGenerator[utils.ModuleData, None]:
    shared_secret = config.parser.get("login", "shared_secret")
    webapi_session = webapi.SteamWebAPI.get_session(0)

    try:
        server_time = cached_server_time()
    except ProcessLookupError:
        yield utils.ModuleData(error=_("Steam is not running."), info=_("Fallbacking server time to WebAPI"))

        try:
            server_time = await webapi_session.get_server_time()
        except aiohttp.ClientError:
            raise aiohttp.ClientError(
                _(
                    "Unable to Connect. You can try these things:\n"
                    "1. Check your connection\n"
                    "2. Check if Steam Server isn't down\n"
                    "3. Check if Steam Client is running\n"
                )
            )

    try:
        if not shared_secret:
            config.new("steamguard", "enable", "false")
            raise ValueError

        auth_code = universe.generate_steam_code(server_time, shared_secret)
    except (ValueError, binascii.Error):
        yield utils.ModuleData(error=_("The current shared secret is invalid."), info=_("Waiting Changes"))
        await asyncio.sleep(10)
    except ProcessLookupError:
        yield utils.ModuleData(status=_("Steam Client is not running"), info=_("Waiting Changes"))
        await asyncio.sleep(10)
    else:
        yield utils.ModuleData(status=_("Loading..."))

        seconds = 30 - (server_time % 30)

        for past_time in range(seconds * 8):
            yield utils.ModuleData(
                display=auth_code,
                status=_("Running"),
                info=_("New code in {} seconds").format(seconds - round(past_time / 8)),
                level=(past_time, seconds * 8),
            )

            await asyncio.sleep(0.125)



#!/usr/bin/env python
#
# Lara Maia <dev@lara.monster> 2015 ~ 2022
#
# The Steam Tools NG is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of
# the License, or (at your option) any later version.
#
# The Steam Tools NG is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see http://www.gnu.org/licenses/.
#
import asyncio
import logging
import sys
from typing import Optional, TYPE_CHECKING

import aiohttp

from stlib import universe, webapi, login
from . import utils
from .. import i18n, config

if TYPE_CHECKING:
    from . import cli

log = logging.getLogger(__name__)
_ = i18n.get_translation


# noinspection PyUnusedLocal
class NewAuthenticator:
    def __init__(self, cli_: 'cli.SteamToolsNG') -> None:
        self.cli = cli_
        self.webapi_session = webapi.SteamWebAPI.get_session(0)
        self._login_data: Optional[login.LoginData] = None
        self._sms_code = ''

    @property
    def sms_code(self) -> str:
        return self._sms_code

    @property
    def oauth_token(self) -> str:
        return config.parser.get("login", "oauth_token")

    @property
    def steamid(self) -> Optional[universe.SteamId]:
        steamid = config.parser.getint("login", "steamid")

        if steamid:
            return universe.generate_steamid(steamid)
        else:
            return None

    async def add_authenticator(self) -> None:
        log.info(_("Retrieving user data"))

        if not self.oauth_token or not self.steamid:
            log.error(_(
                "Some login data is missing. If the problem persists, run:\n"
                "{} --reset"
            ).format(sys.argv[0]))

            self.cli.on_quit(1)

        assert isinstance(self.steamid, universe.SteamId)
        oauth = {'steamid': self.steamid.id64, 'oauth_token': self.oauth_token}
        login_data = login.LoginData(auth={}, oauth=oauth)

        if not self._login_data or not self.sms_code:
            try:
                self._login_data = await self.webapi_session.new_authenticator(self.steamid, self.oauth_token)
            except aiohttp.ClientError:
                log.error(_("Check your connection. (server down?)"))
                self.cli.on_quit(1)
            except webapi.AuthenticatorExists:
                log.error(_(
                    "There's already an authenticator active for that account.\n"
                    "Remove your current steam authenticator and try again."
                ))
                self.cli.on_quit()
            except webapi.PhoneNotRegistered:
                log.error(_(
                    "You must have a phone registered on your steam account to proceed.\n"
                    "Go to your Steam Account Settings, add a Phone Number, and try again."
                ))
                self.cli.on_quit(1)
            except NotImplementedError as exception:
                log.critical("%s: %s", type(exception).__name__, str(exception))
                self.cli.on_quit(1)
            else:
                user_input = utils.safe_input(_("Enter the code received by SMS"))
                assert isinstance(user_input, str)
                self._sms_code = user_input

        log.info(_("Adding authenticator"))
        assert isinstance(self._login_data, login.LoginData)

        try:
            await self.webapi_session.add_authenticator(
                self.steamid,
                self.oauth_token,
                self._login_data.auth['shared_secret'],
                self.sms_code,
            )
        except webapi.SMSCodeError:
            log.error(_("Invalid SMS Code. Please, check the code and try again."))
            self.cli.on_quit(1)
        except aiohttp.ClientError:
            log.error(_("Check your connection. (server down?)"))
            self.cli.on_quit(1)
        except Exception as exception:
            log.critical("%s: %s", type(exception).__name__, str(exception))
            self.cli.on_quit(1)

        log.info(_("Saving new secrets"))
        config.new("login", "shared_secret", self._login_data.auth['shared_secret'])
        config.new("login", "identity_secret", self._login_data.auth['identity_secret'])
        config.new("steamguard", "enable", True)
        config.new("confirmations", "enable", True)

        log.info(_(
            "RECOVERY CODE\n\n"
            "You will need this code to recovery your Steam Account\n"
            "if you lose access to STNG Authenticator. So, write"
            "down this recovery code.\n\n"
            "YOU WILL NOT ABLE TO VIEW IT AGAIN!\n"
        ))

        revocation_code = self._login_data.auth['revocation_code']
        log.info(revocation_code)
        await asyncio.sleep(30)



#!/usr/bin/env python
#
# Lara Maia <dev@lara.monster> 2015 ~ 2022
#
# The Steam Tools NG is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of
# the License, or (at your option) any later version.
#
# The Steam Tools NG is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see http://www.gnu.org/licenses/.
#

import argparse
import configparser
import logging
import os
import sys
from multiprocessing import freeze_support
from pathlib import Path
from subprocess import call
from typing import Optional, Any

from steam_tools_ng import config, i18n
from steam_tools_ng.gtk import application
from steam_tools_ng.gtk import async_gtk, utils

_ = i18n.get_translation
log = logging.getLogger(__name__)


class GraphicalArgParser(argparse.ArgumentParser):
    def _print_message(self, message: str, file: Optional[Any] = None) -> None:
        if message:
            utils.fatal_error_dialog(Exception(message), [])


def main() -> None:
    freeze_support()
    try:
        config.init()
    except configparser.Error as exception:
        utils.fatal_error_dialog(exception, [])
        sys.exit(1)

    command_parser = GraphicalArgParser(formatter_class=argparse.RawDescriptionHelpFormatter)

    command_parser.add_argument(
        '--config-dir',
        action='store_true',
        help='Shows directory used to save config files',
        dest='config_dir'
    )

    command_parser.add_argument(
        '--log-dir',
        action='store_true',
        help='Shows directory used to save log files',
        dest='log_dir',
    )

    command_parser.add_argument(
        '--reset',
        action='store_true',
        help='Clean up settings and log files',
        dest='reset',
    )

    command_parser.add_argument(
        '--reset-password',
        action='store_true',
        help='Clean up saved password',
        dest='reset_password',
    )

    console_params = command_parser.parse_args()

    if console_params.config_dir:
        call(f'{config.file_manager} {config.config_file_directory}')
        sys.exit(0)

    if console_params.log_dir:
        call(f'{config.file_manager} {config.parser.get("logger", "log_directory")}')
        sys.exit(0)

    try:
        config.init_logger()
    except configparser.Error as exception:
        utils.fatal_error_dialog(exception, [])
        sys.exit(1)

    if console_params.reset:
        config.config_file.unlink(missing_ok=True)
        logging.root.removeHandler(logging.root.handlers[0])

        log_directory = config.parser.get("logger", "log_directory")
        Path(log_directory, 'steam-tools-ng.log').unlink()
        Path(log_directory, 'steam-tools-ng.log.1').unlink()

        log.info(_('Done!'))
        sys.exit(0)

    if console_params.reset_password:
        config.new("login", "password", "")
        log.info(_('Done!'))
        sys.exit(0)

    if sys.platform.startswith("linux") and not os.getenv('DISPLAY'):
        log.critical('The DISPLAY is not set!')
        log.critical("Use 'steam-tools-ng' for the command line interface.")
        sys.exit(1)

    app = application.SteamToolsNG()
    async_gtk.run(app)


if __name__ == "__main__":
    main()



#!/usr/bin/env python
#
# Lara Maia <dev@lara.monster> 2015 ~ 2022
#
# The Steam Tools NG is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of
# the License, or (at your option) any later version.
#
# The Steam Tools NG is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see http://www.gnu.org/licenses/.
#

import argparse
import logging
import sys
import textwrap
from multiprocessing import freeze_support
from pathlib import Path

from steam_tools_ng import config, i18n, __version__
from steam_tools_ng.console import cli

_ = i18n.get_translation
log = logging.getLogger(__name__)


def main() -> None:
    freeze_support()
    config.init()

    command_parser = argparse.ArgumentParser(
        formatter_class=argparse.RawDescriptionHelpFormatter,
        epilog=textwrap.dedent('''
                Available modules | available options
                 steamguard       | [oneshot]
                 steamtrades      | [oneshot]
                 steamgifts       | [oneshot]
                 cardfarming      | [oneshot],[gameid]
                 fakerun          | <gameid>
                       '''))

    command_parser.add_argument(
        'module',
        choices=['steamguard', 'steamtrades', 'steamgifts', 'cardfarming', 'fakerun'],
        metavar='<module>',
        action='store',
        nargs='?',
        help='Start a module',
    )

    command_parser.add_argument(
        '--reset',
        action='store_true',
        help='Clean up settings and log files',
        dest='reset',
    )

    command_parser.add_argument(
        '--reset-password',
        action='store_true',
        help='Clean up saved password',
        dest='reset_password',
    )

    command_parser.add_argument(
        '--add-authenticator',
        action='store_true',
        help='Use STNG as your Steam Authenticator',
        dest='add_authenticator',
    )

    command_parser.add_argument(
        '-v', '--version',
        action='store_true',
        help='Show version',
        dest='version',
    )

    command_parser.add_argument(
        'options',
        metavar='<options>',
        nargs='*',
        help=argparse.SUPPRESS,
    )

    console_params = command_parser.parse_args()
    config.init_logger()

    if console_params.version:
        print(__version__)
        sys.exit(0)

    if console_params.reset:
        config.config_file.unlink(missing_ok=True)
        logging.root.removeHandler(logging.root.handlers[0])

        log_directory = config.parser.get("logger", "log_directory")
        Path(log_directory, 'steam-tools-ng.log').unlink()
        Path(log_directory, 'steam-tools-ng.log.1').unlink()

        log.info(_('Done!'))
        sys.exit(0)

    if console_params.reset_password:
        config.new("login", "password", "")
        log.info(_('Done!'))
        sys.exit(0)

    print(f'Steam Tools NG version {__version__} (Made with Girl Power <33)')
    print('Copyright (C) 2015 ~ 2022 Lara Maia - <dev@lara.monster>')

    if console_params.add_authenticator:
        app = cli.SteamToolsNG('add_authenticator', '')
        app.run()
        sys.exit(0)

    if not console_params.module:
        log.critical('No module has scheduled to run.')
        log.critical("Use 'steam-tools-ng-gui' for the graphical user interface.")
        sys.exit(1)

    module_name = console_params.module
    module_options = console_params.options

    app = cli.SteamToolsNG(module_name, module_options)
    app.run()


if __name__ == "__main__":
    main()
